/**
 * @file
 *
 * @brief Describes tty devices interface and structures
 *
 * @date 13.11.2010
 * @author Anton Bondarev
 */

#ifndef TTY_H_
#define TTY_H_

#if 0
#define CONFIG_TTY_WITH_VTPARSE
#endif

#include <types.h>

#define TTY_RXBUFF_SIZE 0x80
#define TTY_TXBUFF_SIZE 0x80

#include <fs/file.h>
#ifdef CONFIG_TTY_WITH_VTPARSE
# include <drivers/vtbuild.h>
# include <drivers/vtparse.h>
#endif

typedef volatile struct tty_device {
	uint8_t  out_buff[TTY_RXBUFF_SIZE + 1];
#ifdef CONFIG_TTY_WITH_VTPARSE
	struct vt_token rx_buff[TTY_RXBUFF_SIZE + 1];
#else
	uint8_t  rx_buff[TTY_RXBUFF_SIZE + 1];
#endif
	uint8_t  tx_buff[TTY_TXBUFF_SIZE + 1];
	uint32_t rx_cnt;
	bool     out_busy; /*whether out_buff is busy*/
	file_operations_t *file_op;
} tty_device_t;

extern tty_device_t *cur_tty;

extern int tty_register(tty_device_t *tty);

extern int tty_unregister(tty_device_t *tty);

extern int tty_get_uniq_number(void);

/*
 * add parsed char to receive buffer
 */
extern int tty_add_char(tty_device_t *tty, int ch);

extern uint8_t* tty_readline(tty_device_t *tty);

extern void tty_freeline(tty_device_t *tty, uint8_t *line);

#endif /* TTY_H_ */
